{-----------------------------------------------------------------------
-- This file is written as part of nudruff.
-- Author: Stephan Beyer
-- Descr: basic binary search tree module, type (_, Integer)
-----------------------------------------------------------------------}
module BSTree (BSTree(Nd, Lf), list2bst, insert, del, delmaybe, maxlq, minOfBST) where

data BSTree a = Lf | Nd (BSTree a) a (BSTree a) deriving (Show, Eq)

-- insert x into BST
insert (f,s) Lf = Nd Lf (f,s) Lf
insert (f,s) (Nd l (mf,ms) r)
	| s <  ms = (Nd (insert (f,s) l) (mf,ms) r)
	| s >= ms = (Nd l (mf,ms) (insert (f,s) r))

-- converts a list into a BST
list2bst ls = foldl (flip insert) Lf ls

-- return smallest element key in BST
minOfBST (Nd l m r)
	| l == Lf   = m
	| otherwise = minOfBST l

-- deletes the node of a given BST
delnode Lf           = Lf
delnode (Nd Lf m Lf) = Lf
delnode (Nd Lf m r)  = r
delnode (Nd l  m Lf) = l
delnode (Nd l  m r)  = (Nd l (minOfBST r) (del (minOfBST r) r))

-- deletes all elements with key `x' out of BST
del x Lf = Lf
del (xf,xs) (Nd l (mf,ms) r)
	| xs < ms   = (Nd (del (xf,xs) l) (mf,ms) r)
	| xs > ms   = (Nd l (mf,ms) (del (xf,xs) r))
	| otherwise = delnode (Nd l (mf,ms) r)

delmaybe Nothing  tree = tree
delmaybe (Just x) tree = del x tree

-- get max element less equal than
-- return value: (maybe maxelement, tree without maxelement)
maxlq size tree = maxleq size tree Nothing
 where	maxleq _ Lf max = (max, delmaybe max tree) 
	maxleq s (Nd l (mf,ms) r) max
		| s <  ms = maxleq s l max
		| s >= ms = maxleq s r (Just (mf,ms))
