{-----------------------------------------------------------------------
-- This file is written as part of nudruff.
-- Author: Stephan Beyer
-- Descr: the bestfit algorithm based on the binary search tree data
-----------------------------------------------------------------------}
module BestFit (bestfit) where
import BSTree (BSTree(Nd,Lf), list2bst, maxlq, minOfBST)

nothingp Nothing = True
nothingp _       = False

unmaybe = maybe undefined

-- flatten :: [[a]] -> [a]
flatten' []      new = new
flatten' (x:old) new = flatten' old new++x
flatten old = flatten' old []

-- creates ((contentlist, contentsize), resttree)
cd' origsize size tree list
	| nothingp e = ((list, origsize-size), tree)  -- t == tree
	| otherwise  = cd' origsize newsize t ((unmaybe fst e):list)
 where	et = maxlq size tree
	e  = fst et -- maybe element of type (name, size)
	t  = snd et -- rest tree
	newsize = size - (unmaybe snd e)

-- returns ((flatcontentlist, contentsize), resttree)
cd size tree = ((flatten (fst f), snd f), snd x)
 where	x = cd' size size tree []
 	f = fst x

-- tail recursive bst -> specific list conversion, order doesn't matter
bst2list Lf             lspair = lspair
bst2list (Nd l (m,s) r) (list,size) = bst2list l (m++fst rp, s+snd rp)
 where	rp = bst2list r (list,size)

cds' size tree list
	| isfinished = (list, bst2list resttree contlist)
	| otherwise  = cds' size resttree (contlist:list)
 where	ltpair     = cd size tree
	contlist   = fst ltpair
	resttree   = snd ltpair
	isfinished = resttree == Lf || snd (minOfBST resttree) > size

-- :: size -> list-of-files -> (list-of-(flatcontentlist,contentsize)-elements, resttree)
bestfit size flist = cds' size (list2bst flist) []
