{-----------------------------------------------------------------------
-- nudruff - an optimal (bestfit) file chooser for archives           --
--     Copyright (C) Stephan Beyer <s-beyer@gmx.net>, 2006, GNU GPLv2 --
-----------------------------------------------------------------------}

module Main (main) where
import ReadConfig (readConfigFile)
import BestFit (bestfit)
import System.Environment (getArgs)
import System.Exit (exitFailure)

printCopyright = do
	putStrLn "nudruff 20060308\t\t\t(C) Stephan Beyer, 2006\n"

printInformation size = do
	putStrLn ("Archive size: "++(show size))

printEach []    = do 
	putStrLn ""
printEach (x:r) = do
	putStrLn x
	printEach r

printArchives ([],                     (rest,size)) _ = do
	putStrLn ("\nREST (no archive, "++(show size)++" bytes)\n")
	printEach rest
printArchives ((current,size):archives,rest)        i = do
	putStrLn ("\nARCHIVE "++(show i)++" ("++(show size)++")\n")
	printEach current
	printArchives (archives,rest) (i+1)

usage =	"Usage: nudruff <size in bytes> <config file>\n\n"++
	"Config file contains each file in each line.\n"++
	"You may group files using `{' and `}' on a single line. No nesting possible.\n"++
	"Grouped files will always be put in the same archive.\n"++
	"You may set a new working directory using `DIR /foo/bar'\n"++
	"See the documentation for more information.\n\nHave fun,\nsbeyer <s-beyer@gmx.net>"

parseArgs = do
	args <- getArgs
	retpair args
 where	argp [s,c] = (s,c)
	retpair args
 	  | length args==2 = do return (read (fst ar)::Integer, snd ar)
	  | otherwise      = do putStrLn usage ; exitFailure
	 where	ar = argp args

main = do
	printCopyright
	(size,configfile) <- parseArgs
	filelist <- readConfigFile configfile
	printInformation size
	-- putStrLn (show (bestfit size filelist))
	printArchives (bestfit size filelist) 1
