{-----------------------------------------------------------------------
-- This file is written as part of nudruff.
-- Author: Stephan Beyer
-- Descr: module to parse and generate a list based upon the configfile
-----------------------------------------------------------------------}
module ReadConfig (readConfigFile) where
import System.IO (openFile, IOMode(ReadMode), hFileSize, hGetLine, hIsEOF, hClose)
import System.Directory (doesFileExist)

-- grouping (begin end)
beginGroup (parsed, False, dir) = (([],0):parsed, True,  dir)
endGroup   (parsed, True,  dir) = (       parsed, False, dir)
-- directory changer
parse ('D':('I':('R':(' ':newdir)))) _ (parsed, grouped, dir) =
	(parsed, grouped, newdir)
-- ungrouped lines:
parse file filesize (parsed, False, dir) =
	(([file], filesize):parsed, False, dir)
-- grouped lines:
parse file filesize ((list,size):parsed, True, dir) =
	((file:list, size+filesize):parsed, True, dir)

-- returns size of file specified by filename (String)
-- or 0 if file is dir or doesn't exist
fileSize fname = do
	b <- doesFileExist fname
	if b then
		do
			h <- openFile fname ReadMode
			size <- hFileSize h
			hClose h
			return size
		else return 0

parseLine h line (parsed, grouped, dir) = do
	size <- fileSize (dir++line)
	readEachLine h (parse line size (parsed, grouped, dir))

readEachLine h pline = do
	eof <- hIsEOF h
	if eof then
		return pline
		else
		do
			line <- hGetLine h
			case line of
				"{" -> readEachLine h (beginGroup pline)
				"}" -> readEachLine h (endGroup pline)
				otherwise -> parseLine h line pline

-- readConfigFile returns a list of ([groupedfilenames], size-of-these-files)
-- if a file doesn't exist or is a directory 0 is the size of those files
readConfigFile configfile = do
	h <- openFile configfile ReadMode
	(x,_,_) <- readEachLine h ([], False, "")
	hClose h
	return x
